==> nono-0.4.4.tar.gz: nono-0.4.4/vm/sysclk.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// LUNA システムクロック
//

// システムクロックは実時間または仮想時間のいずれかに同期するため、
// システムクロック独自の時刻 stime を持つ。
// stime は、割り込みを発生させるごとに、LUNA-I なら 16.67msec、
// LUNA-88K なら 10msec 進行する。

#include "sysclk.h"
#include "config.h"
#include "interrupt.h"
#include "rtc.h"
#include "scheduler.h"
#include "sync.h"

// グローバル参照用
SysClkDevice *gSysClk;

// コンストラクタ
SysClkDevice::SysClkDevice()
	: inherited("SystemClock")
{
	AddAlias("SysClk");

	monitor.func = ToMonitorCallback(&SysClkDevice::MonitorUpdate);
	monitor.SetSize(31, 2);
	monitor.Regist(ID_MONITOR_SYSCLK);

	event.func = ToEventCallback(&SysClkDevice::Callback);
	event.code = 0;
	event.Regist("System Clock");
}

// デストラクタ
SysClkDevice::~SysClkDevice()
{
	gSysClk = NULL;
}

bool
SysClkDevice::Init()
{
	// クロックは機種ごとに異なる
	const ConfigItem& item = gConfig->Find("!sysclk-period");
	period = item.AsInt();
	event.time = period;

	// クロックの同期方法
	sync_rt = gRTC->GetSyncRealtime();

	return true;
}

// 電源オン/リセット
void
SysClkDevice::ResetHard(bool poweron)
{
	// XXX たぶん電源オンだけで作用する?
	if (poweron) {
		if (sync_rt) {
			stime = gSync->GetRealTime();
		}

		// 電源オンですぐ起動
		sysint = false;
		gScheduler->RestartEvent(event);
	}
}

uint64
SysClkDevice::Read(uint32 offset)
{
	// 1 ポートしかない
	return GetInt();
}

uint64
SysClkDevice::Write(uint32 offset, uint32 data)
{
	// 何か書き込まれたら、フラグをクリア
	putlog(3, "Clear system clock interrupt");
	sysint = false;
	gInterrupt->NegateINT(this);
	return 0;
}

uint64
SysClkDevice::Peek(uint32 offset)
{
	return GetInt();
}

void
SysClkDevice::MonitorUpdate(Monitor *, TextScreen& screen)
{
	int y;

	screen.Clear();
	y = 0;

	int width = stime < 86400_sec ? 20 : 25;
	screen.Print(0, y++, "Sync: %s", sync_rt ? "Real Time" : "Virtual Time");
	screen.Print(0, y++, "Time: %*s", width, TimeToStr(stime).c_str());
}

// 割り込み状態を取得(?)
uint8
SysClkDevice::GetInt() const
{
	uint8 data = 0;

	if (sysint) {
		data |= 0x80;
	}
	return data;
}

// イベントハンドラ
void
SysClkDevice::Callback(Event& ev)
{
	putlog(3, "Interrupt occured");

	// システムクロックとしての現在時刻。
	// 足す前は前回の割り込み発生時刻なので period を足すと現在時刻。
	stime += period;

	sysint = true;
	gInterrupt->AssertINT(this);

	if (sync_rt) {
		gScheduler->StartRealtimeEvent(ev, stime, period);
	} else {
		gScheduler->StartEvent(ev);
	}
}

==> nono-0.6.3.tar.gz: nono-0.6.3/vm/sysclk.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// LUNA システムクロック
//

// システムクロックは実時間または仮想時間のいずれかに同期するため、
// システムクロック独自の時刻 stime を持つ。
// stime は、割り込みを発生させるごとに、LUNA-I なら 16.67msec、
// LUNA-88K なら 10msec 進行する。

#include "sysclk.h"
#include "config.h"
#include "interrupt.h"
#include "rtc.h"
#include "scheduler.h"
#include "syncer.h"

// コンストラクタ
SysClkDevice::SysClkDevice()
	: inherited(OBJ_SYSCLK)
{
	monitor.func = ToMonitorCallback(&SysClkDevice::MonitorUpdate);
	monitor.SetSize(31, 3);
	monitor.Regist(ID_MONITOR_SYSCLK);

	event.func = ToEventCallback(&SysClkDevice::Callback);
	event.code = 0;
	event.Regist("System Clock");
}

// デストラクタ
SysClkDevice::~SysClkDevice()
{
}

// 初期化
bool
SysClkDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	interrupt = GetInterruptDevice();

	// クロックは機種ごとに異なる
	const ConfigItem& item = gConfig->Find("!sysclk-freq");
	freq = item.AsInt();
	period = 1_sec / freq;
	event.time = period;

	// クロックの同期方法
	sync_rt = GetRTCDevice()->GetSyncRealtime();

	return true;
}

// 電源オン/リセット
void
SysClkDevice::ResetHard(bool poweron)
{
	// XXX たぶん電源オンだけで作用する?
	if (poweron) {
		if (sync_rt) {
			stime = GetSyncer()->GetRealTime();
		}

		// 電源オンですぐ起動
		sysint = false;
		scheduler->RestartEvent(event);
	}
}

uint64
SysClkDevice::Read(uint32 offset)
{
	// 1 ポートしかない
	return GetInt();
}

uint64
SysClkDevice::Write(uint32 offset, uint32 data)
{
	// 何か書き込まれたら、フラグをクリア
	putlog(3, "Clear system clock interrupt");
	sysint = false;
	interrupt->NegateINT(this);
	return 0;
}

uint64
SysClkDevice::Peek(uint32 offset)
{
	return GetInt();
}

void
SysClkDevice::MonitorUpdate(Monitor *, TextScreen& screen)
{
	int y;

	screen.Clear();
	y = 0;

	int width = stime < 86400_sec ? 20 : 25;
	screen.Print(0, y++, "Freq: %d Hz", freq);
	screen.Print(0, y++, "Sync: %s", sync_rt ? "Real Time" : "Virtual Time");
	screen.Print(0, y++, "Time: %*s", width, TimeToStr(stime).c_str());
}

// 割り込み状態を取得(?)
uint8
SysClkDevice::GetInt() const
{
	uint8 data = 0;

	if (sysint) {
		data |= 0x80;
	}
	return data;
}

// イベントハンドラ
void
SysClkDevice::Callback(Event& ev)
{
	putlog(3, "Interrupt occured");

	// システムクロックとしての現在時刻。
	// 足す前は前回の割り込み発生時刻なので period を足すと現在時刻。
	stime += period;

	sysint = true;
	interrupt->AssertINT(this);

	if (sync_rt) {
		scheduler->StartRealtimeEvent(ev, stime, period);
	} else {
		scheduler->StartEvent(ev);
	}
}
